| package |
package := Package name: 'POPSocket'.
package paxVersion: 0;
	basicComment: ''.

package basicPackageVersion: ''.

"Add the package scripts"

"Add the class names, loose method names, global names, resource names"
package classNames
	add: #POPSocket;
	add: #SimpleClientSocket;
	add: #SMTPSocket;
	yourself.

package methodNames
	yourself.

package globalNames
	yourself.

package resourceNames
	yourself.

"Binary Global Names"
package binaryGlobalNames: (Set new
	yourself).
"Resource Names"
package allResourceNames: (Set new
	yourself).

"Add the prerequisite names"
package setPrerequisites: (IdentitySet new
	add: 'Dolphin';
	add: 'Sockets Connection';
	yourself).

package!

"Class Definitions"!

Socket subclass: #SimpleClientSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SimpleClientSocket subclass: #POPSocket
	instanceVariableNames: 'userName password numMessages'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
SimpleClientSocket subclass: #SMTPSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
"Loose Methods"!

"End of package definition"!



SimpleClientSocket comment: ''!

SimpleClientSocket guid: (GUID fromString: '{B548E0E2-4D6B-11D6-B032-DCB705350804}')!

!SimpleClientSocket categoriesForClass!Unclassified! !
!SimpleClientSocket methodsFor!

connectToServer
	"Private - Answer the result of connect to the server"

	[ self connect  ] on: SocketError
			       do: [ :error | MessageBox notify: 'Failed to connect to server. ' , error description. ^false ].
	self when: #remoteSocketClosed send: #onSocketDisconnected to: self.
	^true
!

getMultilineResponse
	"Get a multiple line response to the last command. A multiple line 
	response ends with a line containing only a single period (.) character."

	| response done line |
	response := WriteStream on: String new.
	done := false.
	[done]
		whileFalse: [ line := self getResponse.
				   response nextPutAll: line.
				  done := line = ('.' , String lineDelimiter)].
	^response contents!

getResponse
	"Answer a one line String with two line delimiter characters (cr lf)"
	
	| result |
	result := self readStream upTo: Character lf asInteger. "The streaming collection is a ByteArray ;)"
	^result asString , Character lf asString
	!

onSocketDisconnected
	"Private - An event has occurred indicating that the remote socket connection was closed 
	so we force a local disconnect"

	self close.
	"and for test only"
	MessageBox notify: 'Connection to server closed!!'!

sendCommand: aString
	"Send aString command through the socket."

	self sendStringCr: aString!

sendStringCr: aString
	"Private - Send aString with cr at end through the socket. Note that this is a convenience method."

	self writeStream nextPutAll: aString; cr; flush! !
!SimpleClientSocket categoriesFor: #connectToServer!*-unclassified!private! !
!SimpleClientSocket categoriesFor: #getMultilineResponse!*-unclassified!public! !
!SimpleClientSocket categoriesFor: #getResponse!*-unclassified!public! !
!SimpleClientSocket categoriesFor: #onSocketDisconnected!*-unclassified!private! !
!SimpleClientSocket categoriesFor: #sendCommand:!operations!public! !
!SimpleClientSocket categoriesFor: #sendStringCr:!operations!private! !



POPSocket comment: ''!

POPSocket guid: (GUID fromString: '{FBE3D9A0-4A24-11D6-B032-85F761705405}')!

!POPSocket categoriesForClass!MailSockets! !
!POPSocket methodsFor!

connectToPOP
	"Answer the result of  connect the receiver to the POP server"

	| response |
	self connectToServer
		ifFalse: [ ^false ].
	Transcript show: 'Connected to: ' , self address displayString; cr.
	"get a hello message"
	Transcript show: (response := self getResponse); cr.
	(response beginsWith: '+')
		ifFalse: [self close. ^false].
	^self login
	



	!

deleteAllMessages
	"Delete all messages"

	1 to: self numMessages do: [:num | self deleteMessage: num]!

deleteMessage: number 
	"Delete the numbered message"

	self sendCommand: 'DELE ' , number printString.
!

disconnectFromPOP
	"Send the quit command to the server and close the receiver"

	self sendCommand: 'QUIT'.
	Transcript show: 'Quit enviado...'; cr.
	self close!

login
	"Private - Answer the result of login to a POP server"

	| response |
	self sendCommand: 'USER ' , userName.
	Transcript show: (response := self getResponse); cr.
	(response beginsWith: '+')
		ifFalse: [self close. ^false].
	self sendCommand: 'PASS ' , password.
	Transcript show: (response := self getResponse); cr.
	(response beginsWith: '+')
		ifFalse: [self close. ^ false].
	^true!

messagesDo: aBlock 
	"Perform aBlock on each message text"

	| thisMessage |
	1 to: self numMessages
	  do: [:num | thisMessage := self retrieveMessage: num.
			aBlock value: thisMessage]!

numMessages
	"Query the server and answer the number of messages that are in the 
	user's mailbox."

	| response answerString |
	numMessages notNil
		ifTrue: [^ numMessages].
	self sendCommand: 'STAT'.
	response := self getResponse.
	Transcript show: 'Respuesta: ', response; cr.
	(response beginsWith: '+OK')
		ifFalse: [ ^0 ].
	answerString := response subStrings second.
	
	numMessages := answerString asNumber asInteger.
	^numMessages!

password
	"Answer the password value"

	^password!

password: aString
	"Set the password"

	password := aString
!

retrieveMessage: number 
	"Retrieve the numbered message"

	| response |
	self sendCommand: 'RETR ' , number printString.
	Transcript show: 'Command: RETR ' , number printString , ' sent ...'; cr.
	response := self getResponse.
	Transcript show: 'Received: ' , response; cr.
	(response beginsWith: '+OK')
		ifFalse: [self error: 'error: ' , response].
	^self getMultilineResponse!

userName
	"Answer the userName value"

	^userName!

userName: aString
	"Set the username"

	userName := aString! !
!POPSocket categoriesFor: #connectToPOP!*-unclassified!public! !
!POPSocket categoriesFor: #deleteAllMessages!*-unclassified!public! !
!POPSocket categoriesFor: #deleteMessage:!*-unclassified!public! !
!POPSocket categoriesFor: #disconnectFromPOP!*-unclassified!public! !
!POPSocket categoriesFor: #login!*-unclassified!private! !
!POPSocket categoriesFor: #messagesDo:!*-unclassified!public! !
!POPSocket categoriesFor: #numMessages!*-unclassified!public! !
!POPSocket categoriesFor: #password!*-unclassified!public! !
!POPSocket categoriesFor: #password:!*-unclassified!public! !
!POPSocket categoriesFor: #retrieveMessage:!*-unclassified!public! !
!POPSocket categoriesFor: #userName!*-unclassified!public! !
!POPSocket categoriesFor: #userName:!*-unclassified!public! !

!POPSocket class methodsFor!

example
	"This is a fast test"

	| pop messages |
	pop := self port: 110 host: (Prompter prompt: 'POP server name').
	pop userName: (Prompter prompt: 'POP user name').
	pop password: (Prompter prompt: 'Password').
	messages := OrderedCollection new.

	pop connectToPOP.
	pop messagesDo: [ :each | messages add: each ].
	"pop deleteAllMessages." "dont touch, only if you want to loose your inbox"
	pop disconnectFromPOP.
	messages inspect.


	! !
!POPSocket class categoriesFor: #example!*-unclassified!public! !



SMTPSocket comment: ''!

SMTPSocket guid: (GUID fromString: '{B548E0E1-4D6B-11D6-B032-DCB705350804}')!

!SMTPSocket categoriesForClass!MailSockets! !
!SMTPSocket methodsFor!

checkSMTPResponse
	"Get an SMTP response, and check that it's in the 200's or 300's. If it's 
	not, close the socket and issue an error:"

	(#(2 3 ) includes: self getSMTPResponse // 100)
		ifFalse: [self close.
			    self error: 'Server responded with an error']!

connectToSMTPServer
	"Connect the receiver to the server"

	self connectToServer
		ifFalse: [ ^false ].
	Transcript show: 'Connected to: ' , self address displayString; cr.
	self checkSMTPResponse.
	self sendCommand: 'HELO aDolphin4.xSystem'.
	self checkSMTPResponse.
	^true!

data: messageData 
	"Send the data of a message"
	"Inform the server we are sending the message data"

	| cookedLine |
	self sendCommand: 'DATA'.
	self checkSMTPResponse.
	"process the data one line at a time"
	messageData lines
			do: [:messageLine | 
				cookedLine := messageLine.
				(cookedLine beginsWith: '.')
					ifTrue: ["lines beginning with a dot must have the dot doubled"
						   cookedLine := '.' , cookedLine].
				self sendCommand: cookedLine].
	"inform the server the entire message text has arrived"
	self sendCommand: '.'.
	self checkSMTPResponse!

getSMTPResponse
	"Wait for an SMTP response, and return the number of the response"

	| line |
	[ line := self getResponse.
	 Transcript show: line.
	 (line at: 4)  = $- ] whileTrue.
	^(line copyFrom: 1 to: 3) asNumber!

mailFrom: fromAddress 
	"Send the command 'MAIL FROM: ' to the server"

	self sendCommand: 'MAIL FROM: <' , fromAddress , '>'.
	self checkSMTPResponse!

mailFrom: sender to: recipientList text: messageText 
	"Deliver this mail to a list of users. NOTE: the recipient list should be a 
	collection of simple internet style addresses -- no '<>' or '()' stuff"

	self mailFrom: sender.
	recipientList do: [:recipient | self recipient: recipient].
	self data: messageText!

quit
	"Send a QUIT command. This is polite to do, and indeed some servers might 
	drop messages that don't have an associated QUIT"

	self sendCommand: 'QUIT'.
	self checkSMTPResponse!

recipient: aRecipient 
	"Specify a recipient for the message. aRecipient should be a bare email 
	address "

	self sendCommand: 'RCPT TO: <' , aRecipient , '>'.
	self checkSMTPResponse! !
!SMTPSocket categoriesFor: #checkSMTPResponse!*-unclassified!public! !
!SMTPSocket categoriesFor: #connectToSMTPServer!*-unclassified!public! !
!SMTPSocket categoriesFor: #data:!*-unclassified!public! !
!SMTPSocket categoriesFor: #getSMTPResponse!*-unclassified!public! !
!SMTPSocket categoriesFor: #mailFrom:!*-unclassified!public! !
!SMTPSocket categoriesFor: #mailFrom:to:text:!*-unclassified!public! !
!SMTPSocket categoriesFor: #quit!*-unclassified!public! !
!SMTPSocket categoriesFor: #recipient:!*-unclassified!public! !

!SMTPSocket class methodsFor!

deliverMailFrom: fromAddress to: recipientList text: messageText usingServer: serverName 
	"Deliver a single email to a list of users and then close the connection. 
	For delivering multiple messages, it is best to create a single connection 
	and send all mail over it. NOTE: the recipient list should be a collection 
	of simple internet style addresses -- no '<>' or '()' stuff"

	| sock |
	sock := self usingServer: serverName.
	sock	mailFrom: fromAddress
		to: recipientList
		text: messageText.
	sock quit.
	sock close.
	^true!

example
	"A simple fast test sending messages to yourself"

	| sock |
	self deliverMailFrom: 'yourName@yourDomain'
		to: #( 'destinationName@destinationDomain' 'destinationName2@destinationDomain2')
		text: 'From: yourName
To: receiverName
Subject: Stupid Test!!

Hello from Dolphin guys :)'
		usingServer: 'yourSMTPserver'!

usingServer: serverName 
	"Create and answer a SMTP socket to the specified server for sending one or more 
	mail messages"

	| sock |
	sock := self port: 25 host: serverName.
	sock connectToSMTPServer.
	^sock! !
!SMTPSocket class categoriesFor: #deliverMailFrom:to:text:usingServer:!*-unclassified!public! !
!SMTPSocket class categoriesFor: #example!*-unclassified!public! !
!SMTPSocket class categoriesFor: #usingServer:!*-unclassified!public! !

 
"Binary Globals"!

"Resources"!

